***********************************************************************
* This macro computes agreement statistics on CCC(rc), precision(r),
* accuracy(ca), TDI(k), CP, and their one sided confidence limits.
* They are computed when target values are random or fixed, and when the
* error is constant or proportional.  It also produce the agreement plot
* around the identity line.  

* Avoid using dataset names of c, cc, t, tt, tb, and p, because these dataset names 
* are used in this macro.

* Written by Lawrence Lin and verified by Min Yang.
* Updated by Lawrence Lin on 6/14/2007

* Validated by Yue Yu through independent SAS programming effort on 3/19/2009, 
* and had written a validation plan and report archived as TR09M-0150 at Baxter SES.
***********************************************************************;

* Parameters are:;
*********************************************************************
	Dataset: name of your data.  We must avoid using dataset names of c, cc, t, tt, tb, and p, 
    because these dataset names are used in this macro.
	Y: reading for the test assay that will be shown in the vertical axis of the agreement plot.
	V_label: label for the vertical axis of the agreement plot
	X: reading for the target assay that will be shown in the horizontal axis of the agreement plot.
	H_label: label for the horizontal axis (target) of the agreement plot
	Min: minimum of the plotting range
	Max: maximum of the plotting range
	By: 
		o	For the constant error structure, this is increment of the plotting range.  For example, by=5.
		o	For the proportional error structure, this is the log scale increments in between min and max.  
			For example, if data range is 1-60, then min=1, max=64, by=2, 4, 8, 16, 32. 
	Error: constant or proportional error
		o	For the proportional error structure, error=const.  Here, TDI is expressed as an absolute difference.
		o	For the proportional error structure, error=prop.  Here, TDI is expressed as a percent change. 
			The natural log transformation to the data will be applied.
	CCC_a: a CCC allowance, which can be set as missing if there is no pre-specified allowance
	CP_a: a CP allowance which must be specified for computing TDI
	TDI_a: a TDI allowance which must be specified for computing CP, 
		and must be a percent value when error=prop is specified 
		or an absolute difference when error=const is specified.
	Target: random or fixed.  
	Alpha: 100(1-alpha)% one-tailed confidence limit.  The default is 0.05.  
	Dec: significant digits after the decimal point printed for TDI when the error=const is specified.  
	For example, if we want to have two significant digit printed, then dec=2.  The default is dec=2.
***********************************************************************;

%macro agreement(dataset=, y=, x=, V_label=, H_label=, min=, max=, by=, 
       CCC_a=., CP_a=, TDI_a=, error=, target=, dec=, alpha=0.05);

**** Set ods escape character to insert simple formatting text into ODS output         ****; 
ods escapechar='^';
%***** Create symbol for turning on subscript. ******;
%LET SUBS=%str(^\sub);

data c; 
 set &dataset; 
 dummy=1;
 y=&y; x=&x; 
 if y=. or x=. then delete;
 if upcase("&error")="PROP" then do;
   y=log(y); x=log(x); 
 end; 
 d=y-x;
 run; 
proc means mean var uss n noprint; var y x d; 
output out=t mean=m1 m2 yyy var=v1 v2 yy uss=xx xxx e2 n=xxxx xxxxx n;
run;

data t; 
 set t; 
 call symputx('obs', n);
 dummy=1;
 mu_d=m1-m2; d2=mu_d**2;
 e2=e2/n; sd2=(e2-d2)*n/(n-3); s12=v1*(n-1)/n; s22=v2*(n-1)/n; 
 U=mu_d/sqrt(SQRT(s12*S22)); V=sqrt(S12/S22); Ca=2/(V+1/V+U**2); 
 rc=1-e2/(d2+s12+s22); 
 r=(rc/ca); 
 d2_s2=(d2/sd2); b1=r*v; b0=m1-b1*m2; 
 se2=s12*(1-r**2)*n/(n-3); e2=e2*n/(n-1);
 kp=&TDI_a;
 if upcase("&error")="PROP" then kp=log(&TDI_a/100+1);

* For CP when the target value is fixed;
%if %upcase("&target")="FIXED" %then %do;
 data cc; merge c t; by dummy;
 di=b0+(b1-1)*x; d2i=di**2;
 cpi=probchi(kp**2/se2,1,d2i/se2); 
 se=sqrt(se2); kpm=(kp+di)/se; kmm=(kp-di)/se;
 C0=pdf('normal',-kpm)-pdf('normal',kmm); C1=C0*x;
 C2=-kpm*pdf('normal',-kpm)-kmm*pdf('normal',kmm);
 run;
 proc means noprint mean; 
  var cpi C0 C1 C2; 
  output out=cpf mean=cp C0 C1 C2;
  run;

 data t; merge t cpf;
  s_p=C0**2+(C0*m2-C1)**2/s22+C2**2/2;
  if cp<1 then s_T=sqrt((s_p/((n-3)*cp**2*(1-cp)**2))); else s_t=.;  
  if cp<1 then T=log(cp/(1-cp)); else T=.;
  T_l=T-probit(1-&alpha)*s_T;
  cp_l=exp(T_l)/(1+exp(T_l));
 run;
%end;

data tt; 
 set t; 
 label rc='CCC' rc_l='Lower CL of CCC'
       r='Precision' r_l='Lower CL of Precision'
       ca='Accuracy' ca_l='Lower CL of Accuracy'
       e2='MSD' e2_u='Upper CL of MSD'
       k='TDI' k_u='Upper CL of TDI'
       cp='CP' cp_l='Lower CL of CP' d2_s2='Relative Bias Square';

 * For CCC, accuracy and precision;
 L_ca=log(ca/(1-ca));
 z_rc=LOG((1+RC)/(1-RC))/2; 
 z_r=LOG((1+R)/(1-R))/2; 

 * For MSD & approx TDI & CP;
 W=LOG(e2);
 k=probit(1-(1-&CP_a)/2)*sqrt(e2);
 cp_a=probchi(kp**2/e2,1); 

 * Standard errors of CCC, accuracy, precision, and MSD;
 if upcase("&target")="RANDOM" then do;
  SZ_rc=((1-R**2)*RC**2/(1-RC**2)/R**2 + 2*RC**3*(1-RC)*U**2/R/(1-RC**2)**2
     - RC**4*U**4/R**2/(1-RC**2)**2/2)/(n-2);
  SZ_r=1/(n-3);
  SL_ca=((ca**2*u**2*(v+1/v-2*r) + ca**2*(v**2+1/v**2+2*r**2)/2
     + (1+r**2)*(ca*u**2-1))/((n-2)*(1-ca)**2));
  s_W=2*(1-d2**2/e2**2)/(n-2);
 end;
 
 if upcase("&target")="FIXED" then do;
  SZ_rc=(1-r**2)*rc**2/((n-2)*(1-rc**2)**2*r**2)*(v*u**2*rc**2+(1-rc*r*v)**2
     + v**2*rc**2*(1-r**2)/2); 
  SZ_r=(1-r**2/2)/(n-3);
  SL_ca=(u**2*v*ca**2*(1-r**2) + (1-v*ca)**2*(1-r**4)/2)/((n-2)*(1-ca)**2);
  s_W=2*(1-(d2+s22*(1-b1)**2)**2/e2**2)/(n-2);
* Add case when there is only one X level. 7/18/03;
  if s22=0 then s_W=2*(1-d2**2/e2**2)/(n-2);
 end;

 SZ_rc=SQRT(SZ_rc); SZ_r=SQRT(SZ_r); SL_ca=SQRT(SL_ca); S_W=sqrt(S_W);
 z_rc_l=z_rc-probit(1-&alpha)*sz_rc; 
 z_r_l=z_r-probit(1-&alpha)*sz_r;
 l_ca_l=L_ca-probit(1-&alpha)*sl_ca;
 rc_l=tanh(z_rc_l); r_l=tanh(z_r_l); ca_l=exp(l_ca_l)/(1+exp(l_ca_l));

 W_u=W+probit(1-&alpha)*s_W;
 e2_u=exp(W_u);
 k_u=probit(1-(1-&CP_a)/2)*sqrt(e2_u);
 cp_a_l=probchi(kp**2/e2_u,1); 
 if upcase("&error")="PROP" then do; k=100*(exp(k)-1); k_u=100*(exp(k_u)-1); end;

* For CP when the target value is random;;
 if upcase("&target")="RANDOM" then do;
  cp=probchi(kp**2/sd2,1,d2_s2); 
  sd=sqrt(sd2); kpm=(kp+mu_d)/sd; kmm=(kp-mu_d)/sd;
  if cp<1 then 
  s_T=((pdf('normal',-kpm)-pdf('normal',kmm))**2
     + (kmm*pdf('normal',kmm)+kpm*pdf('normal',-kpm))**2/2)
     /((n-3)*cp**2*(1-cp)**2); else s_t=.;  
  s_T=sqrt(s_T);
  if cp<1 then T=log(cp/(1-cp)); else T=.;
  T_l=T-probit(1-&alpha)*s_T;
  cp_l=exp(T_l)/(1+exp(T_l));
 end;
 conf=(1-&alpha)*100;
 call symputx('conf', conf);
 keep rc rc_l r r_l ca ca_l e2 e2_u k k_u cp cp_l d2_s2;
*proc print data=tt;
run;

data tb;
 set tt;
 S='Estimate            ';
 CCC=rc; Precision=r; Accuracy=ca; TDI=k; CP=cp; RBS=d2_s2; output;
 S="&conf.% Conf. Limit";
 CCC=rc_l; Precision=r_l; Accuracy=ca_l; TDI=k_u; CP=cp_l; RBS=.; output;
 S='Allowance';
 CCC=&CCC_a; Precision=.; Accuracy=.; TDI=&TDI_a; CP=&CP_a; RBS=.; output;
keep s ccc Precision Accuracy TDI CP RBS;
run;

proc report data=tb headline headskip split='|' nowindows;
  column s CCC Precision Accuracy TDI CP RBS;
 define s/ 'Statistics' format=$20. left;
 define CCC/ 'CCC' format=10.4 right;
 define Precision / 'Precision|Coefficient' format=10.4 right;
 define Accuracy / 'Accuracy|Coefficient' format=10.4 right;
  %if %upcase("&error")="CONST" %then %do;
   define TDI / "TDI &subs &CP_a" format=10.&dec right;
   define CP / "CP &subs &TDI_a" format=10.4 right;
  %end;
  %else %do;
   define TDI / "TDI% &subs &CP_a" format=10.2 right;
   define CP / "CP &subs &TDI_a.%" format=10.4 right;
  %end;
  define RBS / "RBS*" format=10.2 right;
  compute after _page_;
  line "n=&obs";
  line "*: The relative bias squared (RBS) must be less than 1 or 8 for CP_a of 0.9 or 0.8,";
  line "respectively, in order for the approximated TDI to be valid.";
  line "Otherwise, the TDI estimate is conservative depending on the RBS value.";
  endcomp;
 run;

ods rtf startpage=no;
* For graph;
AXIS1 label=(a=90 "&V_label") offset=(1 cm) order=&min to &max by &by ;
AXIS2 label=("&H_label") offset=(1 cm) order=&min to &max by &by ;
AXIS3 label=(a=90 "&V_label") offset=(1 cm) logbase=2 order=&min &by &max ;
AXIS4 label=("&H_label") offset=(1 cm) logbase=2 order=&min &by &max ;

data p; 
 set c;
%if %upcase("&error")="CONST" %then %do; %let na=1; %let nb=2; %end;
%if %upcase("&error")="PROP" %then %do; %let na=3; %let nb=4; %end;
 y0=.; if _n_=1 then y0=&min; if _n_=2 then y0=&max; 
 if upcase("&error")="PROP" then do; y=exp(y); x=exp(x); end; 
PROC GPLOT;
PLOT y*x y0*y0/
overlay FR NOLEGEND VAXIS=AXIS&na HAXIS=AXIS&nb; 
SYMBOL1 h=2 c=black V=circle I=NONE;
SYMBOL2 W=1 c=black V=' ' h= I=join l=1;
run;

ods rtf startpage=now;
%mend agreement;

* The macro calling examples for the JASA paper are provided below:.;
/*
libname ex 'g:\common\lawrence\Agreement Stat\book\Springer Verlag\data\Chapter 2';
filename ex2_2 'g:\common\lawrence\Agreement Stat\book\Springer Verlag\data\Chapter 2\Example2_2.dat';
%inc 'G:\Common\macro\Agreement.sas';

options mprint mlogic symbolgen spool nodate nonumber ls=128 ps=42;
goptions reset=all ftext=swiss htext=1.5 device=sasemf; 
title ' ';

*** example 1 ***;
data e2_1;
  set ex.example2_1;
  y=mean(of hemocue1 hemocue2);
  x=mean(of sigma1 sigma2);
run;
ods rtf file='g:\common\lawrence\Agreement Stat\book\Springer Verlag\rtf\exapmple 2.1.rtf' 
    style=styles.TablesRTF NOTOC_DATA;
%agreement(dataset=e2_1, y=y, x=x, V_label=Hemacue, H_label=Sigma, min=0, max=2000, by=250,
           CCC_a=0.9775, CP_a=0.9, TDI_a=150, 
           error=const, target=random, dec=1, alpha=0.05); 
ods rtf close;

* Example 2;
data e2_2;
 infile ex2_2;
 input x Y1 Y2;
 run;
ods rtf file='g:\common\lawrence\Agreement Stat\book\Springer Verlag\rtf\exapmple 2.2.rtf' 
    style=styles.TablesRTF NOTOC_DATA;
%agreement(dataset=e2_2,y=Y1, x=x, V_label=1:5 FVIII %, H_label=Actual FVIII %, min=1, max=256,
            by=2 4 8 16 32 64 128, 
            CCC_a=0.9775, CP_a=0.8, TDI_a=50,
            error=prop, target=fixed, alpha=0.05); 
%agreement(dataset=e2_2,y=Y2, x=x, V_label=1:10 FVIII %, H_label=Actual FVIII %, min=1, max=256,
            by=2 4 8 16 32 64 128, 
            CCC_a=0.9775, CP_a=0.8, TDI_a=50,
            error=prop, target=fixed, alpha=0.05); 
ods rtf close;

* Example 3;
data e2_3_1;
  set ex.example2_3;
  if acid='Glycine';
data e2_3_2;
  set ex.example2_3;
  if acid='L-methionine';
data e2_3_3;
  set ex.example2_3;
  if acid='L-threonine';
data e2_3_4;
  set ex.example2_3;
  if acid='Ornithine';
run;
ods rtf file='g:\common\lawrence\Agreement Stat\book\Springer Verlag\rtf\exapmple 2.3.rtf' 
    style=styles.TablesRTF NOTOC_DATA;
%agreement(dataset=e2_3_1, y=y, x=x, V_label=Glycine % Label Concentration, H_label=Actual % Label Concentration,
            min=75, max=115, by=10,
            CCC_a=0.99, CP_a=0.8, TDI_a=3,
            error=const, target=random, dec=2, alpha=0.05); 
%agreement(dataset=e2_3_2, y=y, x=x, V_label=L-methionine % Label Concentration, H_label=Actual % Label Concentration,
            min=75, max=115, by=10,
            CCC_a=0.99, CP_a=0.8, TDI_a=3,
            error=const, target=random, dec=2, alpha=0.05); 
%agreement(dataset=e2_3_3, y=y, x=x, V_label=L-threonine % Label Concentration, H_label=Actual % Label Concentration,
            min=75, max=115, by=10,
            CCC_a=0.99, CP_a=0.8, TDI_a=3,
            error=const, target=random, dec=2, alpha=0.05); 
%agreement(dataset=e2_3_4, y=y, x=x, V_label=Ornithine % Label Concentration, H_label=Actual % Label Concentration,
            min=75, max=115, by=10,
            CCC_a=0.99, CP_a=0.8, TDI_a=3,
            error=const, target=random, dec=2, alpha=0.05); 
ods rtf close;
quit;

*/
 
