* Macro for comparison model to evaluate individual agreement.
* This macro requires SAS IML.  No plotting will be produced.
* Here, target values are considered to have random errors.

* Written by Dr. Yuqing Tang on 8/5/2010, 
* Modified by Dr. Lawrence Lin on 8/12/2010, 
* Validated by Mr. Yue Yu through independent R programming on 10/xx/2010, 

* Data structure must be one observation per subject ID  
* Method/assay/rater and replicate must be arranged as m1_1-m1_r m2_1-m2_r, ..., mk_1-mk_r, etc.,
* Data set must be balanced.  If not, delete the unbalanced data.

* Parameters are:;
*********************************************************************
	dataset: specify the dataset name.  We must avoid using dataset names of a, b, c, t1, t2, ttt, bt,
			one, final, because these dataset names are used in this macro.
	var: dependent vaiable names to be evaluated. eg., m1_1-m1_r m2_1-m2_r, ..., mk_1-mk_r, etc. 
	k: the number of methods/raters/instruments/assay, etc
	m: the number of replications for each k
	The number of subjects/samples is computed from the macro.
	TIR_test: the selected test raters for calculating TIR, and must input in the format of ('1','2','3',... 'k').
	Here, '1' represents the first rater m columns, '2' represents the second rater m columns, etc.  
	When calculating multiple TIRs, the test raters for calculating each TIR must be separated by "#". 
	For example, when k=3 and if we specify ('1','3')#('1','2','3')#('3')#('2')#('1','2'), 
	then multiple raters will be averaged.
	TIR_ref: the selected reference raters for calculating TIR that correspond to the TIR_test.
	If ref=(all) is specified, then the intra-raters of all raters will be used as the denominator.
	When calculating multiple TIRs, the corresponding reference raters must be separated by "#".  
	For example, use ('2')#(all)#('1','2')#('1')#('1','2').  
	When TIR_ref is not specified as '(all)', each TIR is computed as the total MSD of test vs reference 
	relative to the intra MSD of the reference raters.  
	For the first TIR example, the macro would assess the average of 
	the total MSD of "raters 1 vs 2 and raters 1 vs 3" relative to the intra MSD of "rater 2".  
	For the second TIR example, the macro would assess average of the total MSD of all raters
	relative to the average of intra MSD of all raters.  
	For the third TIR example, the macro would assess the average of the total MSD of
	"raters 3 vs 1 and raters 3 vs 2" relative to the average of intra MSD of "raters 1 and 2".  
	For the fourth TIR example, the macro would assess the total MSD of "raters 2 and 1" 
	relative to the intra MSD of "rater 1".  
	For the fifth TIR example, the macro would assess the total MSD of "raters 1 and 2" 
	relative to the average of intra MSD of "raters 1 and 2".
	IIR_test: the selected test raters for calculating IIR, which must be input in the format of ('1','2','3',... 'k').
	When calculating multiple IIRs, the test raters for calculating each IIR must be separated by "#".  
	For example, when k=3, specify ('1')#('2')#('3')#('1')
	IIR_ref: the selected reference raters for calculating IIR.  
	When calculating multiple IIRs, the corresponding reference raters must be separated by "#".  
	For example, specify ('2','3')#('1','3')#('1','2')#('2').  
	These reference raters must be mutually exclusive from the test raters.
	Error = const for the constant error structure for continuous data.  
		  = prop for the proportional error structure for continuous data.  
			Here, log transformation to data is applied for the continuous data.  
	For categorical data, use Error=const.
	Alpha: 100(1-alpha)% one-tailed upper confidence limit for TIR or two-tailed confidence interval for IIR. 
	The default is 0.05. 
	TIR_a: allowance for TIR. 
**********************************************************************;

%macro TIR_IIR(dataset=, var=, k=, m=, TIR_test=, TIR_ref=, IIR_test=, IIR_ref=, error=, alpha=0.05, TIR_a=.);

ods escapechar='^';

data a; 
 set &dataset;
 id=_n_;
run;

proc sql;
  create table c as
    select max(id) as n	
	  from a;
quit;

data c;
set c;
call symput ("N",  trim(left(n))); *get total number of subjects;
run;

* Converts data with one observation per row containing rater values of 1 to &k, ID values of 1 to n, 
  and rep values of 1 to &m;
data b;
 set a;
 array xx{&k, &m} &var;
 do rater=1 to &k;
   do rep=1 to &m;
     y=xx(rater, rep);
	 output;
   end;
 end;
run;

data t1;
a="&TIR_test";
b=countc(a, ')'); *calculate the number of TIR generated;
call symput ("numT", b);
run; 

%do i=1 %to &numT;
%let TT_&i = %SCAN(&TIR_test, &i, #); *the test raters used for each TIR;
%let TR_&i = %SCAN(&TIR_ref, &i, #); *the reference raters used for each TIR;
%put &&TT_&i;
%put &&TR_&i;
%end;

%do i=1 %to &numT; *calculat TIR for each set of raters;

   *Output the TIR test and reference raters*;
  
   *The following get the selected test and reference rater;
   data TIR_testrater;
   set b;
   if rater in &&TT_&i then output;
   keep rater;
   run;
   proc sort data=TIR_testrater nodupkey;
   by rater;

   data ttt;*find the number of test raters;
   set TIR_testrater;
   *obs=&k*&m*&N;
   obs = put(&k*&m*&N, 6.) ; *calculate total number of observations.
   *obs1=strip(obs, 6.);
   call symput ("TIR_ntest", _n_);
   call symput ("obs3", trim(left(obs)));
   run;
   %put &TIR_ntest;
   *%let obs1=%sysevalf(&k*&m*&N);

 %if &&TR_&i ne (all) %then %do;
*proc print data=bt;
   data TIR_refrater;
   set b;
   if rater in &&TR_&i then output;
   keep rater;
   run;
  proc sort data=TIR_refrater nodupkey;
   by rater;
   run;

   data null;*find the number of reference raters;
   set TIR_refrater;
   call symput ("TIR_nref", _n_);
   run;
   %put &TIR_nref;
%end;

*proc print data=bt;
data one; 
 set b;
 *y=&y;
 *Transformation for continuous data with proportional error; 
 if upcase("&error")="PROP" then y=log(y);
  *ind=(&rater-1)*&m+&rep;
  conf=(1-&alpha)*100;
   call symputx('conf', conf);
 run; 

 proc sort data=one;
 by id rater rep;
 run;

 data y ( keep = y ); set one; run;

proc iml worksize=5000; reset log noprint;
k = &k;
m = &m;
N = &N;
nc = k*m;


USE Y; READ ALL INTO Y;
Y_matrix=j(N,nc,0);*generate the N*nc matrix for raw data;
p=1;
do i=1 to N;
do s=1 to nc;
Y_matrix[i,s]=y[p,1];
p=p+1;
end;
end;

/*Define Ydot U Ydot2 Z from data Y_matrix*/
Ydot=j(N,k,0);
U=j(N,k,0);

do i=1 to n;
do s=1 to k;
Ydot[i,s]=sum(Y_matrix[i,((s-1)*m+1):(s*m)])/m;
U[i,s]=sum((Y_matrix[i,((s-1)*m+1):(s*m)]-Ydot[i,s])##2)/(m-1);
end;
end;

Ydot2=Ydot##2;
Z=j(N,k*(k-1)/2,0);
p=1;
do s=1 to k-1;
do t=s+1 to k;
Z[,p]=Ydot[,s]#Ydot[,t];
p=p+1;
end;
end;

/*calculate estimates of mu_star, sigma2_star, delta2 and rho_mu*/
mustar=j(k,1,0);
sigma2star=j(k,1,0);
delta2=j(k,1,0);
rhomu=j(k*(k-1)/2,1,0);

do s=1 to k;
mustar[s,1]=sum(Ydot[,s])/N;*estimation of mustar;
sigma2star[s,1]=sum(U[,s])/N;*estimation of sigma2star;
delta2[s,1]=sum(Ydot2[,s])/N-(mustar[s,1])**2-sigma2star[s,1]/m;*estimation of delta2;
end;

tempmu1=j(k,k,0);
tempmu2=j(k,k,0);
tempdelta=j(k,k,0);
tempmudelta1=j(k,k,0);
tempmudelta2=j(k,k,0);
tempsigmadelta=j(k,k,0);

do s=1 to k-1;
do t=s+1 to k;
tempmu1[s,t]=mustar[s,]*mustar[t,];
tempmu2[s,t]=(mustar[s,]-mustar[t,])##2;
tempdelta[s,t]=sqrt(delta2[s,]*delta2[t,]);
tempmudelta1[s,t]=mustar[s,]##2*(delta2[t,]+sigma2star[t,]/m);
tempmudelta2[s,t]=mustar[t,]##2*(delta2[s,]+sigma2star[s,]/m);
tempsigmadelta[s,t]=(delta2[s,]+sigma2star[s,])*(delta2[t,]+sigma2star[t,]);
end;
end;

muproc=j(k*(k-1)/2,1,0);
deltaproc=j(k*(k-1)/2,1,0);
musum=j(k*(k-1)/2,1,0);
mudeltaproc1=j(k*(k-1)/2,1,0);
mudeltaproc2=j(k*(k-1)/2,1,0);
sigmadeltaproc=j(k*(k-1)/2,1,0);

p=1;
do s=1 to k-1;
do t=s+1 to k;
muproc[p,1]=mustar[s,]*mustar[t,];
deltaproc[p,1]=sqrt(delta2[s,]*delta2[t,]);
musum[p,1]=(mustar[s,]-mustar[t,])##2;
mudeltaproc1[p,1]=mustar[s,]##2*(delta2[t,]+sigma2star[t,]/m);
mudeltaproc2[p,1]=mustar[t,]##2*(delta2[s,]+sigma2star[s,]/m);
sigmadeltaproc[p,1]=(delta2[s,]+sigma2star[s,])*(delta2[t,]+sigma2star[t,]);
p=p+1;
end;
end;

do s=1 to k*(k-1)/2;
rhomu[s,1]=(sum(Z[,s])/N-muproc[s,1])/deltaproc[s,1];
end;

/*The following calculates the variance matrix H_i for YY_i*/
Di1=i(k);
Ydotdot=Ydot[:,];
vi1=j(k,k,0);
do s=1 to k;
vi1[s,s]=sum((Ydot[,s]-Ydotdot[,s])##2)/(N-1);
end;

Di2=i(k);
Udot=U[:,];
vi2=j(k,k,0);
do s=1 to k;
vi2[s,s]=sum((U[,s]-Udot[,s])##2)/(N-1);
end;

Di3=i(k);
vi3=j(k,k,0);
do s=1 to k;
vi3[s,s]=2*(delta2[s,1]+sigma2star[s,1]/m)##2+4#mustar[s,1]##2#(delta2[s,1]+sigma2star[s,1]/m);
end;

Di4=i(k*(k-1)/2);
vi4=j(k*(k-1)/2,k*(k-1)/2,0);
do s=1 to k*(k-1)/2;
Di4[s,s]=deltaproc[s,1];
vi4[s,s]=(rhomu[s,1]*deltaproc[s,1])##2+2*muproc[s,1]#rhomu[s,1]#deltaproc[s,1]+sigmadeltaproc[s,1]+mudeltaproc1[s,1]+mudeltaproc2[s,1];
end;

/*The following calculates the derivative of the function w.r.t each parameter*/
Gi1=j(k,k,0);
Gi2=2*diag(mustar);
Gi3=i(k)/m;


index=j(2,k*(k-1)/2,0);
p=1;
do s=1 to k-1;
do t=s+1 to k;
index[1,p]=s;
index[2,p]=t;
p=p+1;
end;
end;

Gi4=j(k,k*(k-1)/2,0);
do s=1 to k*(k-1)/2;
Gi4[index[1,s],s]=mustar[index[2,s],1];
Gi4[index[2,s],s]=mustar[index[1,s],1];
end;
run;

Gi6=j(k,k*(k-1)/2,0);
do s=1 to k*(k-1)/2;
Gi6[index[1,s],s]=rhomu[s,1]*sqrt(delta2[index[2,s],1])/(2*sqrt(delta2[index[1,s],1]));
Gi6[index[2,s],s]=rhomu[s,1]*sqrt(delta2[index[1,s],1])/(2*sqrt(delta2[index[2,s],1]));
end;
run;

/*the following calculates the psi matrix*/
Psi=j(3*k+k*(k-1)/2,3*k+k*(k-1)/2,0);
Psi[1:k, 1:k]=N*Di1*inv(vi1)*Di1`;
Psi[k+1:2*k, k+1:2*k]=N*Di2*inv(vi2)*Di2`;
Psi[2*k+1:3*k, 1:k]=N*Di3*inv(vi3)*Gi2`;
Psi[2*k+1:3*k, k+1:2*k]=N*Di3*inv(vi3)*Gi3`;
Psi[2*k+1:3*k, 2*k+1:3*k]=N*Di3*inv(vi3)*di3`;
Psi[3*k+1:3*k+k*(k-1)/2, 1:k]=N*Di4*inv(vi4)*Gi4`;
Psi[3*k+1:3*k+k*(k-1)/2, 2*k+1:3*k]=N*Di4*inv(vi4)*Gi6`;
Psi[3*k+1:3*k+k*(k-1)/2, 3*k+1:3*k+k*(k-1)/2]=N*Di4*inv(vi4)*Di4`;

/*the following calculates the Lambda matrix*/
Lambda=j(3*k+k*(k-1)/2,3*k+k*(k-1)/2,0);
varY=j(k,k,0);
varY2=j(k,k,0);
varU=j(k,k,0);
covYU=j(k,k,0);
covYY2=j(k,k,0);
covUY2=j(k,k,0);
varZ=j(k*(k-1)/2,k*(k-1)/2,0);
covYZ=j(k,k*(k-1)/2,0);
covUZ=j(k,k*(k-1)/2,0);
covY2Z=j(k,k*(k-1)/2,0);

do i=1 to N;
varY=varY+(Ydot[i,]`-mustar)*(Ydot[i,]`-mustar)`;
varU=varU+(U[i,]`-sigma2star)*(U[i,]`-sigma2star)`;
varY2=varY2+(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)*(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)`;
covYU=covYU+(Ydot[i,]`-mustar)*(U[i,]`-sigma2star)`;
covYY2=covYY2+(Ydot[i,]`-mustar)*(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)`;
covUY2=covUY2+(U[i,]`-sigma2star)*(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)`;
varZ=varZ+(Z[i,]`-rhomu#deltaproc-muproc)*(Z[i,]`-rhomu#deltaproc-muproc)`;
covYZ=covYZ+(Ydot[i,]`-mustar)*(Z[i,]`-rhomu#deltaproc-muproc)`;
covUZ=covUZ+(U[i,]`-sigma2star)*(Z[i,]`-rhomu#deltaproc-muproc)`;
covY2Z=covY2Z+(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)*(Z[i,]`-rhomu#deltaproc-muproc)`;
end;

Lambda[1:k,1:k]=Di1*inv(Vi1)*varY*inv(Vi1)*Di1`;
Lambda[1:k,k+1:2*k]=Di1*inv(Vi1)*covYU*inv(Vi2)*Di2`;
Lambda[k+1:2*k,1:k]=Lambda[1:k,k+1:2*k]`;
Lambda[1:k,2*k+1:3*k]=Di1*inv(Vi1)*covYY2*inv(Vi3)*Di3`;
Lambda[2*k+1:3*k,1:k]=Lambda[1:k,2*k+1:3*k]`;
Lambda[1:k,3*k+1:3*k+k*(k-1)/2]=Di1*inv(Vi1)*covYZ*inv(Vi4)*Di4`;
Lambda[3*k+1:3*k+k*(k-1)/2,1:k]=Lambda[1:k,3*k+1:3*k+k*(k-1)/2]`;

Lambda[k+1:2*k,k+1:2*k]=Di2*inv(Vi2)*varU*inv(Vi2)*Di2`;
Lambda[k+1:2*k,2*k+1:3*k]=Di2*inv(Vi2)*covUY2*inv(Vi3)*Di3`;
Lambda[2*k+1:3*k,k+1:2*k]=Lambda[k+1:2*k,2*k+1:3*k]`;
Lambda[k+1:2*k,3*k+1:3*k+k*(k-1)/2]=Di2*inv(Vi2)*covUZ*inv(Vi4)*Di4`;
Lambda[3*k+1:3*k+k*(k-1)/2,k+1:2*k]=Lambda[k+1:2*k,3*k+1:3*k+k*(k-1)/2]`;

Lambda[2*k+1:3*k,2*k+1:3*k]=Di3*inv(Vi3)*varY2*inv(Vi3)*Di3`;
Lambda[2*k+1:3*k,3*k+1:3*k+k*(k-1)/2]=Di3*inv(Vi3)*covY2Z*inv(Vi4)*Di4`;
Lambda[3*k+1:3*k+k*(k-1)/2,2*k+1:3*k]=Lambda[2*k+1:3*k,3*k+1:3*k+k*(k-1)/2]`;
Lambda[3*k+1:3*k+k*(k-1)/2,3*k+1:3*k+k*(k-1)/2]=Di4*inv(Vi4)*varZ*inv(Vi4)*Di4`;

COV=inv(psi)*Lambda*inv(psi)`;*estimates of variance covariance matrix for all parameters;



/*Calculate TIR when the referece is all raters*/
  %if &&TR_&i=(all) %then %do;
USE TIR_testrater; READ ALL INTO testindex;
*print, mustar delta2 sigma2star rhomu COV;

 msdt=j(k,k,0);*matrix msdt calculates msd total between any two raters;
   p=1;
   Do s = 1 to k;  
     do t = s+1 to k;
	 msdt[t,s]=delta2[t,1]+delta2[s,1]+sigma2star[t,1]+sigma2star[s,1]+(mustar[s,1]-mustar[t,1])**2-2*rhomu[p,1]*SQRT(delta2[t,1])*SQRT(delta2[s,1]);
   p=p+1;
end;
   end;
*print,msdt;

   /*selection of msds of test raters for TIR*/
ntest=&TIR_ntest;
total=j(ntest,ntest,0);*matrix for selected msd total;
*print, total;

do s=1 to ntest;
  do t=1 to ntest;
   total[s,t]=msdt[testindex[s,1],testindex[t,1]];
   end;
   end;
   *print, total;


 msdtotal=log(sum(total)/(ntest*(ntest-1)*0.5));*the log transformation of the average of msds;

 msdw=j(k,1,0);*matrix msdw calculates msd within for each rater;
   do s=1 to k;
   msdw[s,1]=2*sigma2star[s,1];
   end;

 within=j(ntest,1,0);
   do s=1 to ntest;
   within[s,1]=msdw[testindex[s,1],1];
   end;

 msdwithin=log(sum(within)/(ntest));*Average of within for test raters;

 TIR=exp(msdtotal-msdwithin);*Individual difference ratio, all average within is used as denominator;
 *print, msdt total sigma2star within;

 Dvector=j(k*(k-1)/2+3*k,1,0);
   Dmustar=j(k,1,0); 
   Dsigma2star=j(k,1,0);
   Ddelta2=j(k,1,0);
   Drhomu=j(k*(k-1)/2,1,0);
   seltestmustar=j(ntest,1,0);
   
   do s=1 to ntest;
   seltestmustar[s,1]=mustar[testindex[s,1],1];
   end;

   testmubar=sum(seltestmustar)/ntest;
   sumallsigma2=sum(within);*sum of 2 times sigma2 for test raters;


/*the following calculates the derivative with respect to mustar*/
   do s=1 to ntest;
   Dmustar[testindex[s,1],1]=4*ntest*(mustar[testindex[s,1],1]-testmubar)/(sumallsigma2*(ntest-1));
   end;
   *print, Dmustar;
   
  
   /*the following calculates the derivateive with respect to sigmastar*/
   do s=1 to ntest;
   Dsigma2star[testindex[s,1],1]=(2*sumallsigma2-2*ntest*sum(total))/(sumallsigma2**2);
   end;
   *print, Dsigma2star;
   
  
 /*the following calculates the derivative with respect to delta2*/
   rhomatrix=j(k,k,0);
   p=1;
   Do s = 1 to k;  
     do t = s+1 to k;
	 rhomatrix[t,s]=rhomu[p,1];
   p=p+1;
end;
end;
*print, rhomatrix;
   
selrho=j(ntest,ntest,0);
do s=1 to ntest;
  do t=1 to ntest;
   selrho[s,t]=rhomatrix[testindex[s,1],testindex[t,1]];
   selrho[t,s]=selrho[s,t];
   end;
   end;
   *print, selrho;

   seltestdelta2=j(ntest,1,0);

   do t=1 to ntest;
   seltestdelta2[t,1]=delta2[testindex[t,1],1];
   end;
   *print, seltestdelta2;

   do s=1 to ntest;
   Ddelta2[testindex[s,1],1]=(2*(ntest-1)-2*selrho[s,]*sqrt(seltestdelta2)/sqrt(delta2[testindex[s,1],1]))/((ntest-1)*sumallsigma2);
   end;
   *print, Ddelta2;
  
    /*the following calculates the derivative with respect to rhomu*/
 Drhomatrix=j(k,k,0);
 do s=1 to ntest;
  do t=1 to ntest;
  Drhomatrix[testindex[s,1],testindex[t,1]]=-4*sqrt(delta2[testindex[s,1],1])*sqrt(delta2[testindex[t,1],1])/((ntest-1)*sumallsigma2);
  end;
  end;
 *print, Drhomatrix delta2;

p=1;
   Do s = 1 to k;  
     do t = s+1 to k;
	 Drhomu[p,1]=Drhomatrix[t,s];
   p=p+1;
end;
end;
 *print, Drhomu;
  /*generate D vector*/
do s=1 to k;
   Dvector[s,1]=Dmustar[s,1]/TIR;
   Dvector[k+s,1]=Dsigma2star[s,1]/TIR;
   Dvector[2*k+s,1]=Ddelta2[s,1]/TIR;
   end;

   do s=1 to k*(k-1)/2;
   Dvector[3*k+s,1]=Drhomu[s,1]/TIR;
   end;
   *print, Dvector;
   
   var_logTIR=Dvector`*cov*Dvector;*Variance for log transformed TIR;
  *print, var_logTIR;
   se_var_logTIR=sqrt(var_logTIR);*standard deviation for log transformed TIR;
   logTIR_lower_limit=log(TIR)-probit(1-&alpha)*se_var_logTIR;* lower limit for log transformed TIR;
   logTIR_upper_limit=log(TIR)+probit(1-&alpha)*se_var_logTIR;* upper limit for log transformed TIR;
   TIR_lower_limit=exp(logTIR_lower_limit);*lower limit for TIR;
   TIR_upper_limit=exp(logTIR_upper_limit);*upper limit for TIR;

%end;

/*calculate TIR when referencer is not all raters*/

%if &&TR_&i ne (all) %then %do;
   USE TIR_testrater; READ ALL INTO testindex;
   USE TIR_refrater; READ ALL INTO refindex;
   *print,testindex refindex;


   /*Total compare to within; */
   rho_inter=rhomu*sqrt(delta2[1,1])*sqrt(delta2[2,1])/(sqrt(delta2[1,1]+sigma2star[1,1]/m)*sqrt(delta2[2,1]+sigma2star[2,1]/m));
    rho_total=rhomu*sqrt(delta2[1,1])*sqrt(delta2[2,1])/(sqrt(delta2[1,1]+sigma2star[1,1])*sqrt(delta2[2,1]+sigma2star[2,1]));
	rho_intra1=delta2[1,1]/(delta2[1,1]+sigma2star[1,1]);
	rho_intra2=delta2[2,1]/(delta2[2,1]+sigma2star[2,1]);
   *print, mustar delta2 sigma2star rhomu rho_intra1 rho_intra2 rho_inter rho_total COV;
   msdt=j(k,k,0);*matrix msdt calculates msd total between any two raters;
   p=1;
   Do s = 1 to k;  
     do t = s+1 to k;
	 msdt[t,s]=delta2[t,1]+delta2[s,1]+sigma2star[t,1]+sigma2star[s,1]+(mustar[s,1]-mustar[t,1])**2-2*rhomu[p,1]*SQRT(delta2[t,1])*SQRT(delta2[s,1]);
   p=p+1;
end;
   end;
*print,msdt;

ntest=&TIR_ntest;
nref=&TIR_nref;
total=j(ntest,nref,0);*matrix for selected msd total;
*print, total;

/*matrix total has all the msd total as its element*/
do s=1 to ntest;
  do t=1 to nref;
   total[s,t]=msdt[testindex[s,1],refindex[t,1]];
   if total[s,t]=0 then total[s,t]=msdt[refindex[t,1],testindex[s,1]];
   end;
   end;
   *print, total;

   msdtotal=log(sum(total)/(ntest*nref));*the log transformation of the average of msds;

   msdw=j(k,1,0);*matrix msdw calculates msd within for each rater;
   do s=1 to k;
   msdw[s,1]=2*sigma2star[s,1];
   end;

   within=j(nref,1,0);
   do s=1 to nref;
   within[s,1]=msdw[refindex[s,1],1];
   end;

   msdwithin=log(sum(within)/(nref));

   TIR=exp(msdtotal-msdwithin);*Individual difference ratio;

   /*the following calculates the D vector */
   Dvector=j(k*(k-1)/2+3*k,1,0);
   Dmustar=j(k,1,0); 
   Dsigma2star=j(k,1,0);
   Ddelta2=j(k,1,0);
   Drhomu=j(k*(k-1)/2,1,0);
   selrefmustar=j(nref,1,0);
   seltestmustar=j(ntest,1,0);
   do s=1 to nref;
   selrefmustar[s,1]=mustar[refindex[s,1],1];
   end;

   do s=1 to ntest;
   seltestmustar[s,1]=mustar[testindex[s,1],1];
   end;

   sumrefmustar=sum(selrefmustar);
   sumtestmustar=sum(seltestmustar);
   sumrefsigma2=sum(within)/2;

   /*the following calculates the derivative with respect to mustar*/
   do s=1 to ntest;
   Dmustar[testindex[s,1],1]=(nref*mustar[testindex[s,1],1]-sumrefmustar)/(ntest*sumrefsigma2);
   end;

   
   do s=1 to nref;
   Dmustar[refindex[s,1],1]=(ntest*mustar[refindex[s,1],1]-sumtestmustar)/(ntest*sumrefsigma2);
   end;
  
   /*the following calculates the derivateive with respect to sigmastar*/
   do s=1 to ntest;
   Dsigma2star[testindex[s,1],1]=nref/(2*ntest*sumrefsigma2);
   end;

   
   do s=1 to nref;
   Dsigma2star[refindex[s,1],1]=-(sum(total)/(ntest*nref)-sumrefsigma2/nref)/(2*sumrefsigma2**2/nref);
   end;

   /*the following calculates the derivative with respect to delta2*/
   rhomatrix=j(k,k,0);
   p=1;
   Do s = 1 to k;  
     do t = s+1 to k;
	 rhomatrix[t,s]=rhomu[p,1];
   p=p+1;
end;
end;
*print, rhomatrix;
   
selrho=j(ntest,nref,0);
do s=1 to ntest;
  do t=1 to nref;
   selrho[s,t]=rhomatrix[testindex[s,1],refindex[t,1]];
   if selrho[s,t]=0 then selrho[s,t]=rhomatrix[refindex[t,1],testindex[s,1]];
   end;
   end;
   *print, selrho;

   selrefdelta2=j(nref,1,0);
   seltestdelta2=j(ntest,1,0);
   do s=1 to nref;
   selrefdelta2[s,1]=delta2[refindex[s,1],1];
   end;

   do t=1 to ntest;
   seltestdelta2[t,1]=delta2[testindex[t,1],1];
   end;


   do s=1 to ntest;
   Ddelta2[testindex[s,1],1]=(nref-selrho[s,]*sqrt(selrefdelta2)/sqrt(delta2[testindex[s,1],1]))/(2*ntest*sumrefsigma2);
   end;
   
   
   do s=1 to nref;
   Ddelta2[refindex[s,1],1]=(ntest-selrho[,s]`*sqrt(seltestdelta2)/sqrt(delta2[refindex[s,1],1]))/(2*ntest*sumrefsigma2);
   end;

   /*the following calculates the derivative with respect to rhomu*/


 Drhomatrix=j(k,k,0);
 do s=1 to ntest;
  do t=1 to nref;
  Drhomatrix[testindex[s,1],refindex[t,1]]=-sqrt(delta2[testindex[s,1],1])*sqrt(delta2[refindex[t,1],1])/(ntest*sumrefsigma2);
  if Drhomatrix[refindex[t,1],testindex[s,1]]=0 then Drhomatrix[refindex[t,1],testindex[s,1]]=Drhomatrix[testindex[s,1],refindex[t,1]];
  end;
  end;


p=1;
   Do s = 1 to k;  
     do t = s+1 to k;
	 Drhomu[p,1]=Drhomatrix[t,s];
   p=p+1;
end;
end;
  /*generate D vector*/
do s=1 to k;
   Dvector[s,1]=Dmustar[s,1]/TIR;
   Dvector[k+s,1]=Dsigma2star[s,1]/TIR;
   Dvector[2*k+s,1]=Ddelta2[s,1]/TIR;
   end;

   do s=1 to k*(k-1)/2;
   Dvector[3*k+s,1]=Drhomu[s,1]/TIR;
   end;
   *print, Dvector;

   var_logTIR=Dvector`*cov*Dvector;*Variance for log transformed TIR;
   se_var_logTIR=sqrt(var_logTIR);*standard deviation for log transformed TIR;
   logTIR_lower_limit=log(TIR)-probit(1-&alpha)*se_var_logTIR;* lower limit for log transformed TIR;
   logTIR_upper_limit=log(TIR)+probit(1-&alpha)*se_var_logTIR;* upper limit for log transformed TIR;
   TIR_lower_limit=exp(logTIR_lower_limit);*lower limit for TIR;
   TIR_upper_limit=exp(logTIR_upper_limit);*upper limit for TIR;

%end;

estimatestir = round(TIR, .001) ||  round(TIR_upper_limit, .001) ||&TIR_a ;
    create output from estimatestir;
     append from estimatestir;

data storagetir;
set output;
call symput ("a", col1);
call symput ("b", col2);
run;

%let TIR_&i=&a;
%let TIR_upper_limit_&i=&b;

%put &&TIR_&i &&TIR_upper_limit_&i;

%end;

data TIR_all1;
%do i=1 %to &numT;
TIR&i=&&TIR_&i;  
%end;

data TIR_all2;
%do i=1 %to &numT;
TIR&i=&&TIR_upper_limit_&i;  
%end;

data TIR_all3;
%do i=1 %to &numT;
TIR&i=&TIR_a;  
%end;
 
data TIR_all;
set	TIR_all1 TIR_all2 TIR_all3;
run;


***********For IIR*******************;

data t2;
p="&IIR_test";
q=countc(p, ')'); *calculate the number of TIR generated;
call symput ("numR", q);
run; 
%put &numR;

%do i=1 %to &numR;
%let RT_&i = %SCAN(&IIR_test, &i, #); *the test raters used for each IIR;
%let RR_&i = %SCAN(&IIR_ref, &i, #); *the reference raters used for each IIR;
%put &&RT_&i;
%put &&RR_&i;
%end;

%do i=1 %to &numR; *calculat TIR for each set of raters;

   *Output the IIR test and reference raters*;
   *The following get the selected test and reference rater;
   data IIR_testrater;
   set b;
   if rater in &&RT_&i then output;
   keep rater;
   run;
   proc sort data=IIR_testrater nodupkey;
   by rater;

   data null;*find the number of test raters;
   set IIR_testrater;
   call symput ("IIR_ntest", _n_);
   run;
   %put &IIR_ntest;


   data IIR_refrater;
   set b;
   if rater in &&RR_&i then output;
   keep rater;
   run;
  proc sort data=IIR_refrater nodupkey;
   by rater;
   run;

   data null;*find the number of reference raters;
   set IIR_refrater;
   call symput ("IIR_nref", _n_);
   run;
   %put &IIR_nref;
   
data one; 
 set b;
 *y=&y;
 *Transformation for continuous data with proportional error; 
 if upcase("&error")="PROP" then y=log(y);
  *ind=(&rater-1)*&m+&rep;
  conf=(1-&alpha)*100;
   call symputx('conf', conf);
 run; 

 proc sort data=one;
 by id rater rep;
 run;

 data y ( keep = y ); set one; run;

proc iml worksize=5000; reset log noprint;
k = &k;
m = &m;
N = &N;
nc = k*m;


USE Y; READ ALL INTO Y;
Y_matrix=j(N,nc,0);*generate the N*nc matrix for raw data;
p=1;
do i=1 to N;
do s=1 to nc;
Y_matrix[i,s]=y[p,1];
p=p+1;
end;
end;

/*Define Ydot U Ydot2 Z from data Y_matrix*/
Ydot=j(N,k,0);
U=j(N,k,0);

do i=1 to n;
do s=1 to k;
Ydot[i,s]=sum(Y_matrix[i,((s-1)*m+1):(s*m)])/m;
U[i,s]=sum((Y_matrix[i,((s-1)*m+1):(s*m)]-Ydot[i,s])##2)/(m-1);
end;
end;

Ydot2=Ydot##2;
Z=j(N,k*(k-1)/2,0);
p=1;
do s=1 to k-1;
do t=s+1 to k;
Z[,p]=Ydot[,s]#Ydot[,t];
p=p+1;
end;
end;

/*calculate estimates of mu_star, sigma2_star, delta2 and rho_mu*/
mustar=j(k,1,0);
sigma2star=j(k,1,0);
delta2=j(k,1,0);
rhomu=j(k*(k-1)/2,1,0);

do s=1 to k;
mustar[s,1]=sum(Ydot[,s])/N;*estimation of mustar;
sigma2star[s,1]=sum(U[,s])/N;*estimation of sigma2star;
delta2[s,1]=sum(Ydot2[,s])/N-(mustar[s,1])**2-sigma2star[s,1]/m;*estimation of delta2;
end;

tempmu1=j(k,k,0);
tempmu2=j(k,k,0);
tempdelta=j(k,k,0);
tempmudelta1=j(k,k,0);
tempmudelta2=j(k,k,0);
tempsigmadelta=j(k,k,0);

do s=1 to k-1;
do t=s+1 to k;
tempmu1[s,t]=mustar[s,]*mustar[t,];
tempmu2[s,t]=(mustar[s,]-mustar[t,])##2;
tempdelta[s,t]=sqrt(delta2[s,]*delta2[t,]);
tempmudelta1[s,t]=mustar[s,]##2*(delta2[t,]+sigma2star[t,]/m);
tempmudelta2[s,t]=mustar[t,]##2*(delta2[s,]+sigma2star[s,]/m);
tempsigmadelta[s,t]=(delta2[s,]+sigma2star[s,])*(delta2[t,]+sigma2star[t,]);
end;
end;

muproc=j(k*(k-1)/2,1,0);
deltaproc=j(k*(k-1)/2,1,0);
musum=j(k*(k-1)/2,1,0);
mudeltaproc1=j(k*(k-1)/2,1,0);
mudeltaproc2=j(k*(k-1)/2,1,0);
sigmadeltaproc=j(k*(k-1)/2,1,0);

p=1;
do s=1 to k-1;
do t=s+1 to k;
muproc[p,1]=mustar[s,]*mustar[t,];
deltaproc[p,1]=sqrt(delta2[s,]*delta2[t,]);
musum[p,1]=(mustar[s,]-mustar[t,])##2;
mudeltaproc1[p,1]=mustar[s,]##2*(delta2[t,]+sigma2star[t,]/m);
mudeltaproc2[p,1]=mustar[t,]##2*(delta2[s,]+sigma2star[s,]/m);
sigmadeltaproc[p,1]=(delta2[s,]+sigma2star[s,])*(delta2[t,]+sigma2star[t,]);
p=p+1;
end;
end;

do s=1 to k*(k-1)/2;
rhomu[s,1]=(sum(Z[,s])/N-muproc[s,1])/deltaproc[s,1];
end;

/*The following calculates the variance matrix H_i for YY_i*/
Di1=i(k);
Ydotdot=Ydot[:,];
vi1=j(k,k,0);
do s=1 to k;
vi1[s,s]=sum((Ydot[,s]-Ydotdot[,s])##2)/(N-1);
end;

Di2=i(k);
Udot=U[:,];
vi2=j(k,k,0);
do s=1 to k;
vi2[s,s]=sum((U[,s]-Udot[,s])##2)/(N-1);
end;

Di3=i(k);
vi3=j(k,k,0);
do s=1 to k;
vi3[s,s]=2*(delta2[s,1]+sigma2star[s,1]/m)##2+4#mustar[s,1]##2#(delta2[s,1]+sigma2star[s,1]/m);
end;

Di4=i(k*(k-1)/2);
vi4=j(k*(k-1)/2,k*(k-1)/2,0);
do s=1 to k*(k-1)/2;
Di4[s,s]=deltaproc[s,1];
vi4[s,s]=(rhomu[s,1]*deltaproc[s,1])##2+2*muproc[s,1]#rhomu[s,1]#deltaproc[s,1]+sigmadeltaproc[s,1]+mudeltaproc1[s,1]+mudeltaproc2[s,1];
end;

/*The following calculates the derivative of the function w.r.t each parameter*/
Gi1=j(k,k,0);
Gi2=2*diag(mustar);
Gi3=i(k)/m;


index=j(2,k*(k-1)/2,0);
p=1;
do s=1 to k-1;
do t=s+1 to k;
index[1,p]=s;
index[2,p]=t;
p=p+1;
end;
end;

Gi4=j(k,k*(k-1)/2,0);
do s=1 to k*(k-1)/2;
Gi4[index[1,s],s]=mustar[index[2,s],1];
Gi4[index[2,s],s]=mustar[index[1,s],1];
end;
run;

Gi6=j(k,k*(k-1)/2,0);
do s=1 to k*(k-1)/2;
Gi6[index[1,s],s]=rhomu[s,1]*sqrt(delta2[index[2,s],1])/(2*sqrt(delta2[index[1,s],1]));
Gi6[index[2,s],s]=rhomu[s,1]*sqrt(delta2[index[1,s],1])/(2*sqrt(delta2[index[2,s],1]));
end;
run;

/*the following calculates the psi matrix*/
Psi=j(3*k+k*(k-1)/2,3*k+k*(k-1)/2,0);
Psi[1:k, 1:k]=N*Di1*inv(vi1)*Di1`;
Psi[k+1:2*k, k+1:2*k]=N*Di2*inv(vi2)*Di2`;
Psi[2*k+1:3*k, 1:k]=N*Di3*inv(vi3)*Gi2`;
Psi[2*k+1:3*k, k+1:2*k]=N*Di3*inv(vi3)*Gi3`;
Psi[2*k+1:3*k, 2*k+1:3*k]=N*Di3*inv(vi3)*di3`;
Psi[3*k+1:3*k+k*(k-1)/2, 1:k]=N*Di4*inv(vi4)*Gi4`;
Psi[3*k+1:3*k+k*(k-1)/2, 2*k+1:3*k]=N*Di4*inv(vi4)*Gi6`;
Psi[3*k+1:3*k+k*(k-1)/2, 3*k+1:3*k+k*(k-1)/2]=N*Di4*inv(vi4)*Di4`;

/*the following calculates the Lambda matrix*/
Lambda=j(3*k+k*(k-1)/2,3*k+k*(k-1)/2,0);
varY=j(k,k,0);
varY2=j(k,k,0);
varU=j(k,k,0);
covYU=j(k,k,0);
covYY2=j(k,k,0);
covUY2=j(k,k,0);
varZ=j(k*(k-1)/2,k*(k-1)/2,0);
covYZ=j(k,k*(k-1)/2,0);
covUZ=j(k,k*(k-1)/2,0);
covY2Z=j(k,k*(k-1)/2,0);

do i=1 to N;
varY=varY+(Ydot[i,]`-mustar)*(Ydot[i,]`-mustar)`;
varU=varU+(U[i,]`-sigma2star)*(U[i,]`-sigma2star)`;
varY2=varY2+(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)*(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)`;
covYU=covYU+(Ydot[i,]`-mustar)*(U[i,]`-sigma2star)`;
covYY2=covYY2+(Ydot[i,]`-mustar)*(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)`;
covUY2=covUY2+(U[i,]`-sigma2star)*(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)`;
varZ=varZ+(Z[i,]`-rhomu#deltaproc-muproc)*(Z[i,]`-rhomu#deltaproc-muproc)`;
covYZ=covYZ+(Ydot[i,]`-mustar)*(Z[i,]`-rhomu#deltaproc-muproc)`;
covUZ=covUZ+(U[i,]`-sigma2star)*(Z[i,]`-rhomu#deltaproc-muproc)`;
covY2Z=covY2Z+(Ydot2[i,]`-delta2-sigma2star/m-mustar##2)*(Z[i,]`-rhomu#deltaproc-muproc)`;
end;

Lambda[1:k,1:k]=Di1*inv(Vi1)*varY*inv(Vi1)*Di1`;
Lambda[1:k,k+1:2*k]=Di1*inv(Vi1)*covYU*inv(Vi2)*Di2`;
Lambda[k+1:2*k,1:k]=Lambda[1:k,k+1:2*k]`;
Lambda[1:k,2*k+1:3*k]=Di1*inv(Vi1)*covYY2*inv(Vi3)*Di3`;
Lambda[2*k+1:3*k,1:k]=Lambda[1:k,2*k+1:3*k]`;
Lambda[1:k,3*k+1:3*k+k*(k-1)/2]=Di1*inv(Vi1)*covYZ*inv(Vi4)*Di4`;
Lambda[3*k+1:3*k+k*(k-1)/2,1:k]=Lambda[1:k,3*k+1:3*k+k*(k-1)/2]`;

Lambda[k+1:2*k,k+1:2*k]=Di2*inv(Vi2)*varU*inv(Vi2)*Di2`;
Lambda[k+1:2*k,2*k+1:3*k]=Di2*inv(Vi2)*covUY2*inv(Vi3)*Di3`;
Lambda[2*k+1:3*k,k+1:2*k]=Lambda[k+1:2*k,2*k+1:3*k]`;
Lambda[k+1:2*k,3*k+1:3*k+k*(k-1)/2]=Di2*inv(Vi2)*covUZ*inv(Vi4)*Di4`;
Lambda[3*k+1:3*k+k*(k-1)/2,k+1:2*k]=Lambda[k+1:2*k,3*k+1:3*k+k*(k-1)/2]`;

Lambda[2*k+1:3*k,2*k+1:3*k]=Di3*inv(Vi3)*varY2*inv(Vi3)*Di3`;
Lambda[2*k+1:3*k,3*k+1:3*k+k*(k-1)/2]=Di3*inv(Vi3)*covY2Z*inv(Vi4)*Di4`;
Lambda[3*k+1:3*k+k*(k-1)/2,2*k+1:3*k]=Lambda[2*k+1:3*k,3*k+1:3*k+k*(k-1)/2]`;
Lambda[3*k+1:3*k+k*(k-1)/2,3*k+1:3*k+k*(k-1)/2]=Di4*inv(Vi4)*varZ*inv(Vi4)*Di4`;

COV=inv(psi)*Lambda*inv(psi)`;*estimates of variance covariance matrix for all parameters;



/*calculate IIR*/
%if %superq(IIR_test)^= and %superq(IIR_ref)^= %then %do;
   USE IIR_testrater; READ ALL INTO testindex;
   USE IIR_refrater; READ ALL INTO refindex;
   ntest=&IIR_ntest;
   nref=&IIR_nref;

   msdw=j(k,1,0);*matrix msdw calculates msd within for each rater;
   do s=1 to k;
   msdw[s,1]=2*sigma2star[s,1];
   end;

   withintest=j(ntest,1,0);
   do s=1 to ntest;
   withintest[s,1]=msdw[testindex[s,1],1];
   end;

   withinref=j(nref,1,0);
   do s=1 to nref;
   withinref[s,1]=msdw[refindex[s,1],1];
   end;

   msdtest=log(sum(withintest)/(ntest));
   msdref=log(sum(withinref)/(nref));
   IIR=exp(msdtest-msdref);*Within difference ratio;
   
   /*D vector for calculating variance of *IIR*/
   Dvector=j(k,1,0);

   sumtestsigma2=sum(withintest)/2;
   sumrefsigma2=sum(withinref)/2;

   /*the following calculates the derivative with respect to sigma2star*/
   do s=1 to ntest;
   Dvector[testindex[s,1],1]=nref/(ntest*sumrefsigma2*IIR);
   end;
   
   do s=1 to nref;
   Dvector[refindex[s,1],1]=-nref*sumtestsigma2/(ntest*sumrefsigma2*sumrefsigma2*IIR);
   end;

   /*variance for IIR*/
   COV_sigma2=j(k,k,0);

   do s=1 to k;
   do t=1 to k;
   COV_sigma2[s,t]=COV[k+s,k+t];
   end;
   end;

   var_logIIR=Dvector`*COV_sigma2*Dvector;*Variance for log transformed IIR;
   se_var_logIIR=sqrt(var_logIIR);*standard deviation for log transformed TIR;
   logIIR_lower_limit=log(IIR)-probit(1-&alpha/2)*se_var_logIIR;* lower limit for log transformed IIR;
   logIIR_upper_limit=log(IIR)+probit(1-&alpha/2)*se_var_logIIR;* upper limit for log transformed IIR;
   IIR_lower_limit=exp(logIIR_lower_limit);*lower limit for IIR;
   IIR_upper_limit=exp(logIIR_upper_limit);*upper limit for IIR;
   *obs=k*m*N; *calculate total number of observations;
%end;

estimatesiir = round(IIR, .001) || round(IIR_lower_limit, .001) || round(IIR_upper_limit, .001) || 1 ;
    create output from estimatesiir;
     append from estimatesiir;

data storageiir;
set output;
call symput ("c", col1);
call symput ("d", col2);
call symput ("e", col3);
run;

%let IIR_&i=&c;
%let IIR_lower_limit_&i=&d;
%let IIR_upper_limit_&i=&e;

%put &&IIR_&i &&IIR_lower_limit_&i &&IIR_upper_limit_&i;

data IIR1;
   set storageiir;
   IIR_lower_upper="("||strip(&&IIR_lower_limit_&i)||", "||strip(&&IIR_upper_limit_&i)||")";
   call symput ("f", IIR_lower_upper);
   run;

%let IIR_lower_upper_&i=&f;
%put &&IIR_lower_upper_&i;

%end;

data IIR_all1;
%do i=1 %to &numR;
length IIR&i $20.;
IIR&i="&&IIR_&i";  
%end;

data IIR_all2;
%do i=1 %to &numR;
IIR&i="&&IIR_lower_upper_&i";  
%end;
run;
 
data IIR_all3;
%do i=1 %to &numR;
IIR&i="1.00";  
%end;

data IIR_all;
set	IIR_all1 IIR_all2 IIR_all3;
run;

data s;
length s $20.;
s='Estimate'; output;
s="&conf.% Conf. Limit*"; output;
s='Compared to'; output;
run;

data final;
merge s TIR_all IIR_all;
totalTIR="TIR"||strip(&numT);
totalIIR="IIR"||strip(&numR);
 call symput ("y", totalTIR);
 call symput ("z", totalIIR);
run;

proc report data=final headline headskip split='|' nowindows;
  column s ('TIR' TIR1-&y) ('IIR' IIR1-&z) ;
 define s/ 'Statistics'  style={just=center cellwidth=2 in};
 %do i=1 %to &numT;
define TIR&i/ "Total^{sub&&TT_&i vs &&TR_&i}/|Intra^{sub&&TR_&i}" style={just=center cellwidth=1.2 in};
 %end;
 %do i=1 %to &numR;
define IIR&i/ "Intra^{sub&&RT_&i}/|Intra^{sub&&RR_&i}" style={just=center cellwidth=1.2 in};
 %end;
 compute after _page_;
  line "n=&N, k=&k, and m=&m";
  line "*: One-tailed upper limit for TIR and two-tailed interval for IIR";
  endcomp;
 run;


%mend TIR_IIR;


/*
*Examples of running the macros;

libname agree 'D:\thesis\July2010';
ods rtf file='D:\thesis\July2010\bp.rtf'; 
   %comparison(dataset=agree.bp, var=J1-J3 R1-R3 S1-S3, k=3, m=3, TIR_test=('3'), TIR_ref=('1','2'),
               IIR_test=('3'), IIR_ref=('1','2'), error=prop, alpha=0.05);
ods rtf close; 	

libname agree 'D:\thesis\July2010';

proc delete data=work._all_; run;
ods rtf file='D:\thesis\July2010\glu.rtf'; 
   %comparison(dataset=agree.gludata, var=f2 f3 f4 f5, k=2, m=2, TIR_test=('1','2'), TIR_ref=all, 
               IIR_test=('1'), IIR_ref=('2'), error=const, alpha=0.05);
ods rtf close; 

libname agree 'D:\thesis\July2010';
ods rtf file='D:\thesis\July2010\bone.rtf'; 
   %comparison(dataset=agree.bone_agree, var=m1_1-m1_2 m2_1-m2_2 m3_1-m3_2, k=3, m=2, TIR_test=('1','3','2'), TIR_ref=all,
               IIR_test=('3'), IIR_ref=('1','2'), error=const, alpha=0.05);
ods rtf close; 

libname agree 'D:\thesis\July2010';
ods rtf file='D:\thesis\July2010\addedibe.rtf'; 
   %comparison(dataset=agree.addedibe, var=t1 t2 r1 r2, k=2, m=2, TIR_test=('1'), TIR_ref=('2'), IIR_test=('1'), IIR_ref=('2'), 
   error=prop, alpha=0.05);
ods rtf close; 	

*/
